﻿using Battleship.Drawings;

namespace Battleship.CollectionGenericObjects;

public abstract class AbstractCompany
{
    /// <summary>
    /// Размер места (ширина)
    /// </summary>
    protected readonly int _placeSizeWidth = 238;

    /// <summary>
    /// Размер места (высота)
    /// </summary>
    protected readonly int _placeSizeHeight = 80;

    /// <summary>
    /// Ширина окна
    /// </summary>
    protected int _pictureWidth;

    /// <summary>
    /// Высота окна
    /// </summary>
    protected int _pictureHeight;

    /// <summary>
    /// Коллекция объектов
    /// </summary>
    protected ICollectionGenericObjects<DrawingWarship>? _collection = null;

    /// <summary>
    /// Вычисление максимального количества элементов, который можно разместить в окне
    /// </summary>
    private int GetMaxCount => _pictureWidth * _pictureHeight / (_placeSizeWidth * _placeSizeHeight) + 2;

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="picWidth">Ширина окна</param>
    /// <param name="picHeight">Высота окна</param>
    /// <param name="collection">Коллеккция кораблей</param>
    public AbstractCompany(int picWidth, int picHeight, ICollectionGenericObjects<DrawingWarship> collection)
    {
        _pictureWidth = picWidth;
        _pictureHeight = picHeight;
        _collection = collection;
        _collection.SetMaxCount = GetMaxCount;
    }

    /// <summary>
    /// Перезагрузка оператора сложения для класса
    /// </summary>
    /// <param name="company">Компания</param>
    /// <param name="car">Добавляемый объект</param>
    /// <returns></returns>
    public static int operator +(AbstractCompany company, DrawingWarship warship)
    {
        return company._collection.Insert(warship);
    }
    /// <summary>
    /// Перезагрузка оператора удаления для класса
    /// </summary>
    /// <param name="company">Компания</param>
    /// <param name="position">Номер удаляемого объекта</param>
    /// <returns></returns>
    public static DrawingWarship operator -(AbstractCompany company, int position) 
    { 
        return company._collection.Remove(position);
    }

    /// <summary>
    /// Получение случайного объекта из коллекции
    /// </summary>
    /// <returns></returns>
    public DrawingWarship? GetRandomObject()
    {
        Random rnd = new();
        return _collection?.Get(rnd.Next(GetMaxCount));
    }

    /// <summary>
    /// Вывод всей коллекции
    /// </summary>
    /// <returns></returns>
    public Bitmap? Show()
    {
        Bitmap bitmap = new(_pictureWidth, _pictureHeight);
        Graphics graphics = Graphics.FromImage(bitmap);
        DrawBackground(graphics);

        SetObjectsPosition();
        for (int i = 0; i < (_collection?.Count ?? 0); ++i)
        {
            DrawingWarship? obj = _collection?.Get(i);
            obj?.DrawTransport(graphics);
        }
        return bitmap;
    }

    /// <summary>
    /// Вывод заднего фона
    /// </summary>
    /// <param name="g"></param>
    protected abstract void DrawBackground(Graphics g);

    /// <summary>
    /// Расстановка объектов
    /// </summary>
    protected abstract void SetObjectsPosition();
}