﻿using Battleship.Drawings;
using Battleship.MovementStrategy;

namespace Battleship;
/// <summary>
/// Форма работы с объектом Линкор
/// </summary>
public partial class FormBattleship : Form
{
    /// <summary>
    /// Поле/объект для прорисовки объекта
    /// </summary>
    private DrawingWarship? _drawingWarship; //поля с нижнего подчеркивания

    /// <summary>
    /// Стратегия перемещения
    /// </summary>
    private AbstractStrategy? _strategy;


    /// <summary>
    /// Получение объекта
    /// </summary>
    public DrawingWarship SetWarship
    {
        set
        {
            _drawingWarship = value;
            _drawingWarship.SetPictureSize(pictureBoxBattleship.Width, pictureBoxBattleship.Height);
            _strategy = null;
            Draw();
        }
    }

    /// <summary>
    /// Конструктор формы
    /// </summary>
    public FormBattleship()
    {
        InitializeComponent();
        _strategy = null;
    }
    /// <summary>
    /// Метод прорисовки машины
    /// </summary>
    //создаем отдельный метод
    private void Draw()
    {
        if (_drawingWarship == null)
        {
            return;
        }

        Bitmap bmp = new(pictureBoxBattleship.Width, pictureBoxBattleship.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawingWarship.DrawTransport(gr);
        pictureBoxBattleship.Image = bmp;

    }

    /// <summary>
    /// кнопки вверх/вниз/влево/вправо 
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonMove_Click(object sender, EventArgs e)
    {
        if (_drawingWarship == null)
        {
            return;
        }

        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawingWarship.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawingWarship.MoveTransport(DirectionType.Down);
                break;
            case "buttonLeft":
                result = _drawingWarship.MoveTransport(DirectionType.Left);
                break;
            case "buttonRight":
                result = _drawingWarship.MoveTransport(DirectionType.Right);
                break;
        }

        if (result)
        {
            Draw();
        }
    }

    private void buttonStrategyStep_Click(object sender, EventArgs e)
    {
        if (_drawingWarship == null)
        {
            return;
        }

        if (comboBoxStrategy.Enabled)
        {
            _strategy = comboBoxStrategy.SelectedIndex
            switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorder(),
                _ => null,
            };

            if (_strategy == null)
            {
                return;
            }
            _strategy.SetData(new MoveableWarship(_drawingWarship),
            pictureBoxBattleship.Width, pictureBoxBattleship.Height);
        }

        if (_strategy == null)
        {
            return;
        }
        comboBoxStrategy.Enabled = false;
        _strategy.MakeStep();
        Draw();

        if (_strategy.GetStatus() == StrategyStatus.Finish)
        {
            comboBoxStrategy.Enabled = true;
            _strategy = null;
        }
    }
}